@@ -43,9 +43,8 @@ from libqtile.command import interface
 from libqtile.command.base import CommandError, CommandException, CommandObject, expose_command
 from libqtile.command.client import InteractiveCommandClient
 from libqtile.command.interface import IPCCommandServer, QtileCommandInterface
-from libqtile.config import Click, Drag, Key, KeyChord, Match, Mouse, Rule
+from libqtile.config import Click, Drag, Key, KeyChord, Match, Mouse, Rule, Screen, ScreenRect
 from libqtile.config import ScratchPad as ScratchPadConfig
-from libqtile.config import Screen, ScreenRect
 from libqtile.core.lifecycle import lifecycle
 from libqtile.core.loop import LoopContext, QtileEventLoopPolicy
 from libqtile.core.state import QtileState
@@ -60,11 +59,13 @@ from libqtile.utils import cancel_tasks, get_cache_dir, lget, remove_dbus_rules,
 from libqtile.widget.base import _Widget
 
 if TYPE_CHECKING:
-    from typing import Any, Callable, Literal
+    from collections.abc import Callable
+    from typing import Any, Literal
 
     from libqtile.command.base import ItemT
     from libqtile.confreader import Config
     from libqtile.layout.base import Layout
+    from libqtile.utils import ColorType
 
 
 class Qtile(CommandObject):
@@ -219,25 +220,29 @@ class Qtile(CommandObject):
         faulthandler.register(signal.SIGUSR2, all_threads=True)
 
         try:
-            async with LoopContext(
-                {
-                    signal.SIGTERM: self.stop,
-                    signal.SIGINT: self.stop,
-                    signal.SIGHUP: self.stop,
-                    signal.SIGUSR1: self.reload_config,
-                }
-            ), ipc.Server(
-                self._prepare_socket_path(self.socket_path),
-                self.server.call,
+            async with (
+                LoopContext(
+                    {
+                        signal.SIGTERM: self.stop,
+                        signal.SIGINT: self.stop,
+                        signal.SIGHUP: self.stop,
+                        signal.SIGUSR1: self.reload_config,
+                    }
+                ),
+                ipc.Server(
+                    self._prepare_socket_path(self.socket_path),
+                    self.server.call,
+                ),
             ):
                 await self._stopped_event.wait()
         finally:
             self.finalize()
             self.core.remove_listener()
 
-    def stop(self) -> None:
+    def stop(self, exitcode: int = 0) -> None:
         hook.fire("shutdown")
         lifecycle.behavior = lifecycle.behavior.TERMINATE
+        lifecycle.exitcode = exitcode
         self.core.graceful_shutdown()
         self._stop()
 
@@ -423,7 +428,7 @@ class Qtile(CommandObject):
             if screen not in screens:
                 for gap in screen.gaps:
                     if isinstance(gap, bar.Bar) and gap.window:
-                        gap.kill_window()
+                        gap.finalize()
 
         self.screens = screens
 
@@ -450,6 +455,9 @@ class Qtile(CommandObject):
     def paint_screen(self, screen: Screen, image_path: str, mode: str | None = None) -> None:
         self.core.painter.paint(screen, image_path, mode)
 
+    def fill_screen(self, screen: Screen, background: ColorType) -> None:
+        self.core.painter.fill(screen, background)
+
     def process_key_event(self, keysym: int, mask: int) -> tuple[Key | KeyChord | None, bool]:
         key = self.keys_map.get((keysym, mask), None)
         if key is None:
@@ -484,8 +492,10 @@ class Qtile(CommandObject):
         Useful when a keyboard mapping event is received.
         """
         self.core.ungrab_keys()
-        for key in self.keys_map.values():
-            self.core.grab_key(key)
+        keys = self.keys_map.copy()
+        self.keys_map.clear()
+        for key in keys.values():
+            self.grab_key(key)
 
     def grab_key(self, key: Key | KeyChord) -> None:
         """Grab the given key event"""
@@ -577,9 +587,12 @@ class Qtile(CommandObject):
         label: str | None = None,
         index: int | None = None,
         screen_affinity: int | None = None,
+        persist: bool | None = False,
     ) -> bool:
         if name not in self.groups_map.keys():
-            g = _Group(name, layout, label=label, screen_affinity=screen_affinity)
+            g = _Group(
+                name, layout, label=label, screen_affinity=screen_affinity, persist=persist
+            )
             if index is None:
                 self.groups.append(g)
             else:
@@ -600,25 +613,28 @@ class Qtile(CommandObject):
         # one group per screen is needed
         if len(self.groups) == len(self.screens):
             raise ValueError("Can't delete all groups.")
+
         if name in self.groups_map.keys():
             group = self.groups_map[name]
-            if group.screen and group.screen.previous_group:
-                target = group.screen.previous_group
-            else:
-                target = group.get_previous_group()
 
-            # Find a group that's not currently on a screen to bring to the
-            # front. This will terminate because of our check above.
-            while target.screen:
-                target = target.get_previous_group()
+            # Find a group that's not currently on a screen to bring to the front.
+            target = group.get_previous_group(skip_managed=True)
+
+            # move windows to other group
             for i in list(group.windows):
                 i.togroup(target.name)
+
+            # if group to be deleted is currently active
             if self.current_group.name == name:
+                # switch to target group
                 self.current_screen.set_group(target, save_prev=False)
+
             self.groups.remove(group)
             del self.groups_map[name]
+
             hook.fire("delgroup", name)
             hook.fire("changegroup")
+
             self.update_desktops()
 
     def register_widget(self, w: _Widget) -> None:
@@ -972,7 +988,7 @@ class Qtile(CommandObject):
 
         return self._eventloop.call_soon_threadsafe(f)
 
-    def call_later(self, delay: int, func: Callable, *args: Any) -> asyncio.TimerHandle:
+    def call_later(self, delay: int | float, func: Callable, *args: Any) -> asyncio.TimerHandle:
         """Another event loop proxy, see `call_soon`."""
 
         def f() -> None:
@@ -1063,7 +1079,7 @@ class Qtile(CommandObject):
 
             def getformat(self) -> tuple[str, int]:
                 format_string = " ".join(
-                    "%-{0:d}s".format(max_col_size + 2) for max_col_size in self.max_col_size
+                    f"%-{max_col_size + 2:d}s" for max_col_size in self.max_col_size
                 )
                 return format_string + "\n", len(self.max_col_size)
 
@@ -1086,7 +1102,12 @@ class Qtile(CommandObject):
 
         def walk_binding(k: Key | KeyChord, mode: str) -> None:
             nonlocal rows
-            modifiers, name = ", ".join(k.modifiers), k.key
+            modifiers = ", ".join(k.modifiers)
+            if isinstance(k.key, int):
+                name = hex(k.key)
+            else:
+                name = k.key
+
             if isinstance(k, Key):
                 if not k.commands:
                     return
@@ -1096,7 +1117,7 @@ class Qtile(CommandObject):
                         for value in k.commands[0].args
                     ]
                     + [
-                        "%s = %s" % (keyword, repr(value))
+                        f"{keyword} = {repr(value)}"
                         for keyword, value in k.commands[0].kwargs.items()
                     ]
                 )
@@ -1105,7 +1126,7 @@ class Qtile(CommandObject):
                         mode,
                         name,
                         modifiers,
-                        "{:s}({:s})".format(k.commands[0].name, allargs),
+                        f"{k.commands[0].name:s}({allargs:s})",
                         k.desc,
                     ]
                 )
@@ -1117,11 +1138,11 @@ class Qtile(CommandObject):
                     if mode == "<root>"
                     else "{}>{}".format(mode, k.name if k.name else "_")
                 )
-                rows.append([mode, name, modifiers, "", "Enter {:s} mode".format(new_mode_s)])
+                rows.append([mode, name, modifiers, "", f"Enter {new_mode_s:s} mode"])
                 for s in k.submappings:
                     walk_binding(s, new_mode)
                 return
-            raise TypeError("Unexpected type: {}".format(type(k)))
+            raise TypeError(f"Unexpected type: {type(k)}")
 
         for k in self.config.keys:
             walk_binding(k, "<root>")
@@ -1396,7 +1417,7 @@ class Qtile(CommandObject):
         return [
             i.info()
             for i in self.windows_map.values()
-            if not isinstance(i, (base.Internal, _Widget)) and isinstance(i, CommandObject)
+            if not isinstance(i, base.Internal | _Widget) and isinstance(i, CommandObject)
         ]
 
     @expose_command()
@@ -1425,9 +1446,16 @@ class Qtile(CommandObject):
         return dictionary
 
     @expose_command()
-    def shutdown(self) -> None:
-        """Quit Qtile"""
-        self.stop()
+    def shutdown(self, exitcode: int = 0) -> None:
+        """Quit Qtile
+
+        Parameters
+        ==========
+        exitcode :
+            Set exit status of Qtile. Can be e.g. used to make login managers
+            poweroff or restart the system. (default: 0)
+        """
+        self.stop(exitcode)
 
     @expose_command()
     def switch_groups(self, namea: str, nameb: str) -> None:
@@ -1658,7 +1686,7 @@ class Qtile(CommandObject):
                     logger.debug("No command entered.")
                     return
                 try:
-                    result = eval("c.{0:s}".format(cmd))
+                    result = eval(f"c.{cmd:s}")
                 except (CommandError, CommandException, AttributeError):
                     logger.exception("Command errored:")
                     result = None
@@ -1667,7 +1695,7 @@ class Qtile(CommandObject):
 
                     message = pformat(result)
                     if messenger:
-                        self.spawn('{0:s} "{1:s}"'.format(messenger, message))
+                        self.spawn(f'{messenger:s} "{message:s}"')
                     logger.debug(result)
 
         mb = self.widgets_map[widget]
@@ -1684,10 +1712,11 @@ class Qtile(CommandObject):
         layout: str | None = None,
         layouts: list[Layout] | None = None,
         index: int | None = None,
+        persist: bool | None = False,
     ) -> bool:
         """Add a group with the given name"""
         return self.add_group(
-            name=group, layout=layout, layouts=layouts, label=label, index=index
+            name=group, layout=layout, layouts=layouts, label=label, index=index, persist=persist
         )
 
     @expose_command()
@@ -1730,6 +1759,7 @@ class Qtile(CommandObject):
     def hide_show_bar(
         self,
         position: Literal["top", "bottom", "left", "right", "all"] = "all",
+        screen: Literal["current", "all"] = "current",
     ) -> None:
         """Toggle visibility of a given bar
 
@@ -1737,16 +1767,28 @@ class Qtile(CommandObject):
         ==========
         position :
             one of: "top", "bottom", "left", "right", or "all" (default: "all")
+        screen :
+            one of: "current", "all" (default: "current")
         """
+        to_mod = [self.current_screen]
+        if screen == "all":
+            to_mod = self.screens
+        for s in to_mod:
+            self.hide_show_bar_screen(s, position)
+
+    def hide_show_bar_screen(
+        self,
+        screen: Screen,
+        position: Literal["top", "bottom", "left", "right", "all"] = "all",
+    ) -> None:
         if position in ["top", "bottom", "left", "right"]:
-            bar = getattr(self.current_screen, position)
+            bar = getattr(screen, position)
             if bar:
                 bar.show(not bar.is_show())
                 self.current_group.layout_all()
             else:
                 logger.warning("Not found bar in position '%s' for hide/show.", position)
         elif position == "all":
-            screen = self.current_screen
             is_show = None
             for bar in [screen.left, screen.right, screen.top, screen.bottom]:
                 if isinstance(bar, libqtile.bar.Bar):
