@@ -33,14 +33,6 @@ from random import randint
 from shutil import which
 from typing import TYPE_CHECKING
 
-if TYPE_CHECKING:
-    from typing import Any, Callable, Coroutine, TypeVar, Union
-
-    ColorType = Union[str, tuple[int, int, int], tuple[int, int, int, float]]
-    ColorsType = Union[ColorType, list[ColorType]]
-
-    T = TypeVar("T")
-
 try:
     from dbus_next import AuthError, Message, Variant
     from dbus_next.aio import MessageBus
@@ -52,6 +44,14 @@ except ImportError:
 
 from libqtile.log_utils import logger
 
+ColorType = str | tuple[int, int, int] | tuple[int, int, int, float]
+ColorsType = ColorType | list[ColorType]
+if TYPE_CHECKING:
+    from collections.abc import Callable, Coroutine
+    from typing import Any, TypeVar
+
+    T = TypeVar("T")
+
 dbus_bus_connections = set()
 
 # Create a list to collect references to tasks so they're not garbage collected
@@ -112,7 +112,7 @@ def rgb(x: ColorType) -> tuple[float, float, float, float]:
 
     Which is returned as (1.0, 0.0, 0.0, 0.5).
     """
-    if isinstance(x, (tuple, list)):
+    if isinstance(x, tuple | list):
         if len(x) == 4:
             alpha = x[-1]
         else:
@@ -142,7 +142,7 @@ def rgb(x: ColorType) -> tuple[float, float, float, float]:
 
 def hex(x: ColorType) -> str:
     r, g, b, _ = rgb(x)
-    return "#%02x%02x%02x" % (int(r * 255), int(g * 255), int(b * 255))
+    return f"#{int(r * 255):02x}{int(g * 255):02x}{int(b * 255):02x}"
 
 
 def has_transparency(colour: ColorsType) -> bool:
@@ -152,7 +152,7 @@ def has_transparency(colour: ColorsType) -> bool:
     Where a list of colours is passed, returns True if any
     colour is not fully opaque.
     """
-    if isinstance(colour, (str, tuple)):
+    if isinstance(colour, str | tuple):
         return rgb(colour)[3] < 1
     return any(has_transparency(c) for c in colour)
 
@@ -161,7 +161,7 @@ def remove_transparency(colour: ColorsType):  # type: ignore
     """
     Returns a tuple of (r, g, b) with no alpha.
     """
-    if isinstance(colour, (str, tuple)):
+    if isinstance(colour, str | tuple):
         return tuple(x * 255.0 for x in rgb(colour)[:3])
     return [remove_transparency(c) for c in colour]
 
@@ -229,7 +229,7 @@ def describe_attributes(obj: Any, attrs: list[str], func: Callable = lambda x: x
     for attr in attrs:
         value = getattr(obj, attr, None)
         if func(value):
-            pairs.append("%s=%s" % (attr, value))
+            pairs.append(f"{attr}={value}")
 
     return ", ".join(pairs)
 
@@ -267,13 +267,13 @@ def lazify_imports(
     """
     __all__ = tuple(registry.keys())
 
-    def __dir__() -> tuple[str, ...]:
+    def __dir__() -> tuple[str, ...]:  # noqa: N807
         return __all__
 
-    def __getattr__(name: str) -> Any:
+    def __getattr__(name: str) -> Any:  # noqa: N807
         if name not in registry:
             raise AttributeError
-        module_path = "{}.{}".format(package, registry[name])
+        module_path = f"{package}.{registry[name]}"
         return import_class(module_path, name, fallback=fallback)
 
     return __all__, __dir__, __getattr__
@@ -427,6 +427,7 @@ async def _send_dbus_message(
     body: Any,
     negotiate_unix_fd: bool = False,
     bus: MessageBus | None = None,
+    preserve: bool = False,
 ) -> tuple[MessageBus | None, Message | None]:
     """
     Private method to send messages to dbus via dbus_next.
@@ -471,7 +472,8 @@ async def _send_dbus_message(
     # Keep details of bus connections so we can close them on exit
     # dbus_bus_connetions is a set so we don't need to worry about
     # duplicates
-    dbus_bus_connections.add(bus)
+    if not preserve:
+        dbus_bus_connections.add(bus)
 
     return bus, msg
 
@@ -484,6 +486,8 @@ async def add_signal_receiver(
     bus_name: str | None = None,
     path: str | None = None,
     check_service: bool = False,
+    use_bus: MessageBus | None = None,
+    preserve: bool = False,
 ) -> bool:
     """
     Helper function which aims to recreate python-dbus's add_signal_receiver
@@ -515,7 +519,7 @@ async def add_signal_receiver(
     }
 
     rule = "type='signal',"
-    rule += ",".join("{}='{}'".format(k, v) for k, v in match_args.items() if v)
+    rule += ",".join(f"{k}='{v}'" for k, v in match_args.items() if v)
 
     logger.debug("Adding dbus match rule: %s", rule)
 
@@ -528,6 +532,8 @@ async def add_signal_receiver(
         "AddMatch",
         "s",
         [rule],
+        bus=use_bus,
+        preserve=preserve,
     )
 
     # Check if message sent successfully
